<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Base class for a template engine.
 * This does <b>nothing</b> and <b>must</b> be
 * derived.
 * See myMVC_SmartyEngine for a template engine
 * using Smarty (http://www.smarty.net/)
 */
abstract class myMVC_BaseEngine
{
  protected $templateDirectory;
  protected $compileDirectory;

  /**
   * Constructor
   * 
   * @param string $templateDirectory   Directory with the templates
   * @param string $compileDirectory    Directory with the compiled templates (from engine)
   * @throws myMVC_MVCException         On problems
   */
  public function __construct($templateDirectory, $compileDirectory)
  {
    if(!is_dir($templateDirectory))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
              sprintf("%s is not directory", $templateDirectory));
      throw new myMVC_MVCException($msg);
    }
    if(!is_readable($templateDirectory))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
              sprintf("Directory %s is not readable", $templateDirectory));
      throw new myMVC_MVCException($msg);
    }
    if(!is_dir($compileDirectory))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
              sprintf("%s is not directory", $compileDirectory));
      throw new myMVC_MVCException($msg);
    }
    if(!is_readable($compileDirectory))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
              sprintf("Directory %s is not readable", $compileDirectory));
      throw new myMVC_MVCException($msg);
    }
    if(!is_writable($compileDirectory))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
              sprintf("Directory %s is not writable", $compileDirectory));
      throw new myMVC_MVCException($msg);
    }
    $this->templateDirectory = $templateDirectory;
    $this->compileDirectory = $compileDirectory;
  }

  /**
   * Returns the template directory
   * 
   * @return string                     The template directory
   */
  public function getTemplateDirectory()
  {
    return $this->templateDirectory;
  }

  /**
   * Assign a key to the template
   * 
   * @param string $key                 The key
   * @param mixed $value                The value
   */
  abstract public function assignVariable($key, $value);

  /**
   * Assign an object to the template (reference)
   * 
   * @param string $key                 The key
   * @param mixed $object               The object
   */
  abstract public function assignObject($key, $object);

  /**
   * Register a function to be called from template
   * 
   * @param string $name                The name of the function (in template!!)
   * @param mixed $func                 The name function or method (method in the format class::method)
   *                                    Also accepted in the format array('class', 'method').
   */
  abstract public function registerFunction($name, $func);

  /**
   * Register a modifier to be called from template
   * 
   * @param string $name                The name of the modifier (in template!!)
   * @param mixed $func                 The name function or method (method in the format class::method)
   *                                    Also accepted in the format array('class', 'method').
   */
  abstract public function registerModifier($name, $func);

  /**
   * Render a template an send it to the browser
   * 
   * @param class $view                 The current view (to get the template)
   */
  abstract public function display($view);

  /**
   * Render a template an returns it as string
   * 
   * @param class $view                 The current view (to get the template)
   * @return string                     The rendered template
   */
  abstract public function fetch($view);
}
?>
