<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Main controller.
 * It manages all requests and responses.
 * Example to use it:
 * 
 * <code>
 * define('SMARTY_DIR',                '../Smarty/libs/');
 * require_once(SMARTY_DIR.'Smarty.class.php');
 * require_once('mymvc/myMVC.php');
 * require_once('../etc/etc.php'); // <b>YOUR</b> define file
 * 
 * $controller = new myMVC_Controller();
 * $controller->setRenderEngine(new myMVC_SmartyEngine($_SERVER['DOCUMENT_ROOT'].'/../pages/templates',
 *                                                     $_SERVER['DOCUMENT_ROOT'].'/../pages/templates_c'));
 * $controller->setActions(array('MainAction', 'TestAction', 'Test1Action', 'NoRightsAction', 'LoginAction', 'myMVC_BaseLogoutAction'),
 *                         'MainAction', 'NoRightsAction');
 * $controller->setViews(array('MainView', 'TestView', 'Test1View', 'NoRightsView', 'LoginView'));
 * $controller->setLanguageInformation(array('de_DE' => 'Deutsch', 'en_US' => 'English', 'it_IT' => 'Italiano'),
 *                                     'de_DE', $_SERVER['DOCUMENT_ROOT'].'/../locale', 'testmvc');
 * $controller->setFilter(myMVC_Controller::$FILTER_PREACTION, 'myMVC_LoginFilter', array(), array('loginAction' => 'LoginAction'));
 * $controller->process();
 * </code>
 */
class myMVC_Controller
{
  private $request;
  private $session;
  private $defaultAction;
  private $noRightAction;
  private $actions;
  private $views;
  private $availableLanguages;
  private $getTextLocaleDir;
  private $getTextLocaleDomain;
  private $languageParameter;
  private $renderEngine;
  private $filters;

  public static $FILTER_PREACTION       = 'preaction';
  public static $FILTER_PREVIEW         = 'preview';

  /**
   * Constructor.
   * Initializes all engines (Request, Session, Template engine, and so on)
   */
  public function __construct()
  {
    global $config;

    $this->request = myMVC_HttpRequest::getInstance();
    $this->session = myMVC_HttpSession::getInstance();

    $this->defaultAction = NULL;
    $this->noRightAction = NULL;
    $this->actions = array ();
    $this->views = array ();
    $this->getTextLocaleDir = '';
    $this->getTextLocaleDomain = '';
    $this->languageParameter = 'lang';
    $this->renderEngine = NULL;
    $this->filters = array(self::$FILTER_PREACTION => NULL, self::$FILTER_PREVIEW => NULL);

    if(isset($config['TIMEZONE']))
      date_default_timezone_set($config['TIMEZONE']);
  }

  /**
   * Set the render engine zu use for template.
   * Currently just myMVC_SmartyEngine is implemented.
   * 
   * @param class $class                    The render engine. It must be a subclass of myMVC_BaseEngine
   */
  public function setRenderEngine($class)
  {
    if(!is_subclass_of($class, 'myMVC_BaseEngine'))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Engine class %s is not a subclass of myMVC_BaseEngine", get_class($class)));
      throw new myMVC_MVCException($msg);
    }
    $this->renderEngine = $class;
  }

  /**
   * Returns a list of possible actions
   * 
   * @return array                          The list of the possible actions, or NULL if no action
   *                                        is registerd
   */
  public static function getActionsList()
  {
    $session = myMVC_HttpSession::getInstance();
    if($session->existsInNamespace('controller', 'actionInfo'))
    {
      $info = $session->getFromNamespace('controller', 'actionInfo');
      return $info['actions'];
    }
    return NULL;
  }

  /**
   * Deletes the currently saved action list.
   * It must be called after a successful login!
   */
  public static function clearActionList()
  {
    $session = myMVC_HttpSession::getInstance();
    $session->deleteFromNamespace('controller', 'actionInfo');
  }

  /**
   * Returns the current defined default action.
   * 
   * @return string                         The class name of the default action or NULL
   *                                        if no default action was defined
   */
  public static function getDefaultAction()
  {
    $session = myMVC_HttpSession::getInstance();
    if($session->existsInNamespace('controller', 'actionInfo'))
    {
      $info = $session->getFromNamespace('controller', 'actionInfo');
      return $info['defaultAction'];
    }
    return NULL;
  }

  /**
   * Returns the current called action.
   * 
   * @return string                         The class name of the current action or NULL
   *                                        if no action was called
   */
  public static function getCurrentAction()
  {
    $session = myMVC_HttpSession::getInstance();
    if($session->existsInNamespace('controller', 'currentAction'))
      return $session->getFromNamespace('controller', 'currentAction');
    return NULL;
  }

  /**
   * Returns the list of all actions, that should appear
   * in the menu.
   * 
   * @return array                          The list of all menu action. It can be empty
   */
  public static function getMenuActions()
  {
    $session = myMVC_HttpSession::getInstance();
    if($session->existsInNamespace('controller', 'menuActions'))
      return $session->getFromNamespace('controller', 'menuActions');
    return array();
  }

  /**
   * Returns whether the user has rights for the given action.
   * 
   * @param string $action                  The class name of the requested action
   * @return boolean                        true if the user has rights to call the action,
   *                                        false otherwise
   */
  public static function hasUserRightForAction($action)
  {
    $session = myMVC_HttpSession::getInstance();
    $actionsWithRight = $session->getFromNamespace('controller', 'userRights');
    return in_array($action, array_keys($actionsWithRight));
  }

  /**
   * Set the available actions list.
   * 
   * @param array $actions                  Array of Action Class names (derived from myMVC_BaseAction)
   * @param string $defaultAction           Name of the default Action (called if no Action was given)
   * @param string $noRightAction           Name of the Action to be called if the current users does not
   *                                        have the rights for the called Action.
   *                                        If it is NULL, and the user doesn't have the rights, an Exception
   *                                        will be generated
   * @throws myMVC_MVCException             On problems
   */
  public function setActions($actions, $defaultAction, $noRightAction = NULL)
  {
    $mustRun = true;
    if($this->session->existsInNamespace('controller', 'actionInfo'))
    {
      $info = $this->session->getFromNamespace('controller', 'actionInfo');
      $this->actions = $info['actions'];
      $this->defaultAction = $info['defaultAction'];
      $this->noRightAction = $info['noRightAction'];
      $mustRun = false;
    }

    if($mustRun)
    {
      myMVC_Logger::doLog(__METHOD__, __LINE__, 'Actions not in session. Parsing them', ALERT_DEBUG);
      if(!is_array($actions))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Given parameter for available actions is not an array');
        throw new myMVC_MVCException($msg);
      }
      if(count($actions) == 0)
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Array with available actions cannot be empty');
        throw new myMVC_MVCException($msg);
      }
      $userRights = array();
      $menuActions = array();
      $this->actions = array ();
      foreach ($actions as $action)
      {
        if(!class_exists($action))
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Action class %s is not defined", $action));
          throw new myMVC_MVCException($msg);
        }
        if(!is_subclass_of($action, 'myMVC_BaseAction'))
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Action class %s is not a subclass of myMVC_BaseAction", $action));
          throw new myMVC_MVCException($msg);
        }
        if(!is_callable(array($action, 'getActionToken')))
          throw new myMVC_MVCException(sprintf("Action class %s has no method getActionToken", $action));
        $this->actions[call_user_func(array($action, 'getActionToken'))] = $action;
        $userRights[$action] = call_user_func(array($action, 'hasUserRightForAction'));
        if(call_user_func(array($action, 'isMenuAction')) &&
           call_user_func(array($action, 'getActionMenuName')) != '' &&
           call_user_func(array($action, 'hasUserRightForAction')))
          array_push($menuActions, $action);
      }
      $this->session->setInNamespace('controller', 'userRights', $userRights);
      $this->session->setInNamespace('controller', 'menuActions', $menuActions);
      if(!is_string($defaultAction) || $defaultAction == '')
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Given parameter for default action is invalid');
        throw new myMVC_MVCException($msg);
      }
      if(!class_exists($defaultAction))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Default action class %s is not defined", $defaultAction));
        throw new myMVC_MVCException($msg);
      }
      if(!is_callable(array($defaultAction, 'getActionToken')))
        throw new myMVC_MVCException(sprintf("Action class %s has no method getActionToken", $defaultAction));
      $this->defaultAction = call_user_func(array($defaultAction, 'getActionToken'));

      if($noRightAction !== NULL && !class_exists($noRightAction))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Action class %s is not defined", $noRightAction));
        throw new myMVC_MVCException($msg);
      }
      if($noRightAction !== NULL)
      {
        if(!is_callable(array($noRightAction, 'getActionToken')))
          throw new myMVC_MVCException(sprintf("Action class %s has no method getActionToken", $noRightAction));
        $this->noRightAction = call_user_func(array($noRightAction, 'getActionToken'));
        myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Using class %s if user hasn't rights", $noRightAction), ALERT_DEBUG);
      }
      else
        $this->noRightAction = NULL;

      $this->session->setInNamespace('controller', 'actionInfo', 
                array('actions' => $this->actions,
                      'defaultAction' => $this->defaultAction,
                      'noRightAction' => $this->noRightAction));
    }
  }

  /**
   * Set the available views list.
   * 
   * @param array $views                    Array of View Class names (derived from myMVC_BaseView)
   * @throws myMVC_MVCException             On problems
   */
  public function setViews($views)
  {
    $mustRun = true;
    if($this->session->existsInNamespace('controller', 'views'))
    {
      $this->views = $this->session->getFromNamespace('controller', 'views');
      $mustRun = false;
    }

    if($mustRun)
    {
      myMVC_Logger::doLog(__METHOD__, __LINE__, 'Views not in session. Parsing them', ALERT_DEBUG);
      if(!is_array($views))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Given parameter for available views is not an array');
        throw new myMVC_MVCException($msg);
      }
      if(count($views) == 0)
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Array with available views cannot be empty');
        throw new myMVC_MVCException($msg);
      }
      $this->views = array();
      foreach ($views as $view)
      {
        if(!class_exists($view))
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("View class %s is not defined", $view));
          throw new myMVC_MVCException($msg);
        }
        if(!is_subclass_of($view, 'myMVC_BaseView'))
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("View class %s is not a subclass of myMVC_BaseView", $view));
          throw new myMVC_MVCException($msg);
        }
        if(!is_callable(array($view, 'getViewToken')))
          throw new myMVC_MVCException(sprintf("View class %s has no method getViewToken", $view));
        $this->views[call_user_func(array (
          $view,
          'getViewToken'
        ))] = $view;
      }
      $this->session->setInNamespace('controller', 'views', $this->views);
    }
  }

  /**
   * Sets the filter for the controller.
   * A filter can be called before an action or a view
   * IMPORTANT: just a filter per type can be defined!!
   * 
   * @param int $type                       Filter type
   *                                        (myMVC_Controller::$FILTER_PREACTION or myMVC_Controller::$FILTER_PREVIEW)
   * @param string $class                   The class of the filter (derived from myMVC_BaseFilter)
   * @param array $exceptions               Optional list of classes (Action or View, depending on filter type,
   *                                        which has to be excluded from filter [aka: the controller will call
   *                                        this class, and not the filter])
   * @param array $params                   Optional parameters for the filter
   * @throws myMVC_MVCException             On problems
   */
  public function setFilter($type, $class, $exceptions = array(), $params = array())
  {
    $mustRun = true;
    if($this->session->existsInNamespace('controller', 'filters'))
    {
      $this->filters = $this->session->getFromNamespace('controller', 'filters');
      $mustRun = false;
    }

    if($mustRun)
    {
      if(!in_array($type, array(self::$FILTER_PREACTION, self::$FILTER_PREVIEW)))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Unknown filter type '%s'", $type));
        throw new myMVC_MVCException($msg);
      }
      if(!class_exists($class))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Filter class %s is not defined", $class));
        throw new myMVC_MVCException($msg);
      }
      if(!is_subclass_of($class, 'myMVC_BaseFilter'))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Filter class %s is not a subclass of myMVC_BaseFilter", $class));
        throw new myMVC_MVCException($msg);
      }
      if(!is_callable(array($class, 'filter')))
        throw new myMVC_MVCException(sprintf("Filter class %s has no method filter", $class));
      $exceptionsClasses = array();
      if($this->noRightAction !== NULL)
        array_push($exceptionsClasses, $this->noRightAction);
      if(is_array($exceptions))
        foreach($exceptions as $exception)
        {
          switch($type)
          {
            case self::$FILTER_PREACTION:
              if(!is_subclass_of($exception, 'myMVC_BaseAction'))
              {
                $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Action class %s is not a subclass of myMVC_BaseAction", $exception));
                throw new myMVC_MVCException($msg);
              }
              if(!is_callable(array($exception, 'getActionToken')))
                throw new myMVC_MVCException(sprintf("Action class %s has no method getActionToken", $exception));
              array_push($exceptionsClasses, call_user_func(array($exception, 'getActionToken')));
            break;
            case self::$FILTER_PREVIEW:
              if(!is_subclass_of($exception, 'myMVC_BaseView'))
              {
                $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("View class %s is not a subclass of myMVC_BaseView", $exception));
                throw new myMVC_MVCException($msg);
              }
              if(!is_callable(array($exception, 'getViewToken')))
                throw new myMVC_MVCException(sprintf("View class %s has no method getViewToken", $exception));
              array_push($exceptionsClasses, call_user_func(array($exception, 'getViewToken')));
            break;
          }
        }
      if($this->filters[$type] !== NULL)
        myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Filter for type %s already defined. Overriding it", $type), ALERT_WARNING);
      else
        myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Using filter %s for %s", $class, $type), ALERT_DEBUG);
      $this->filters[$type] = array('class' => $class, 'params' => $params, 'exceptions' => $exceptionsClasses);
      $this->session->setInNamespace('controller', 'filters', $this->filters);
    }
  }

  /**
   * Set the language information (GetText)
   * 
   * @param array $availableLanguages       Associative array of languages (code => description, eg. 'de_DE' => 'Deutsch')
   * @param string $defaultLanguage         Default language. It MUST be one in the array $availableLanguages
   * @param string $getTextLocaleDir        Directory of the GetText files
   * @param string $getTextLocaleDir        GetText Domain
   * @param string $languageParameter       Name of the parameter with the new language (default: lang)
   * @throws myMVC_MVCException             On problems
   */
  public function setLanguageInformation($availableLanguages, $defaultLanguage, $getTextLocaleDir, $getTextLocaleDomain, $languageParameter = 'lang')
  {
    $mustRun = true;
    if($this->session->existsInNamespace('controller', 'languageInfo'))
    {
      $languageInfo = $this->session->getFromNamespace('controller', 'languageInfo');
      $this->availableLanguages = $languageInfo['availableLanguages'];
      $this->defaultLanguage = $languageInfo['defaultLanguage'];
      $this->getTextLocaleDir = $languageInfo['getTextLocaleDir'];
      $this->getTextLocaleDomain = $languageInfo['getTextLocaleDomain'];
      $this->languageParameter = $languageInfo['languageParameter'];
      $mustRun = false;
    }

    if($mustRun)
    {
      myMVC_Logger::doLog(__METHOD__, __LINE__, 'Language information not in session. Parsing them', ALERT_DEBUG);
      if(!is_array($availableLanguages))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Given parameter for available languages is not an array');
        throw new myMVC_MVCException($msg);
      }
      if(count($availableLanguages) == 0)
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Array with available languages cannot be empty');
        throw new myMVC_MVCException($msg);
      }
      if(!is_string($defaultLanguage) || $defaultLanguage == '')
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Given parameter for default language is invalid');
        throw new myMVC_MVCException($msg);
      }
      if(!isset($availableLanguages[$defaultLanguage]))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Given parameter for default language is not in the list of available languages');
        throw new myMVC_MVCException($msg);
      }
      if(!is_dir($getTextLocaleDir))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Given parameter for locale directory is not a directory');
        throw new myMVC_MVCException($msg);
      }
      if(!is_string($getTextLocaleDomain) || $getTextLocaleDomain == '')
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'Given parameter for locale doman is invalid');
        throw new myMVC_MVCException($msg);
      }
      $this->availableLanguages = $availableLanguages;
      $this->defaultLanguage = $defaultLanguage;
      $this->getTextLocaleDir = $getTextLocaleDir;
      $this->getTextLocaleDomain = $getTextLocaleDomain;
      $this->languageParameter = $languageParameter;
      $this->session->setInNamespace('controller', 'languageInfo', array (
        'availableLanguages' => $this->availableLanguages,
        'defaultLanguage' => $this->defaultLanguage,
        'getTextLocaleDir' => $this->getTextLocaleDir,
        'getTextLocaleDomain' => $this->getTextLocaleDomain,
        'languageParameter' => $this->languageParameter
      ));
    }
  }

  /**
   * Returns all available languages in the program
   * 
   * @return array                          Associative array of languages (code => description, eg. 'de_DE' => 'Deutsch')
   */
  public static function getAvailableLanguages()
  {
    $session = myMVC_HttpSession::getInstance();
    if($session->existsInNamespace('controller', 'languageInfo'))
    {
      $languageInfo = $session->getFromNamespace('controller', 'languageInfo');
      return $languageInfo['availableLanguages'];
    }
    return NULL;
  }

  /**
   * Returns the current used language
   * 
   * @return string                         The current language (eg: de_DE or en_US)
   */
  public static function getCurrentLanguage()
  {
    $session = myMVC_HttpSession::getInstance();
    if($session->existsInNamespace('controller', 'currentLanguage'))
      return str_replace('.UTF-8', '', $session->getFromNamespace('controller', 'currentLanguage'));
    return NULL;
  }

  /**
   * Get the wanted language (from browser or from parameter)
   * and load the language definition (for GetText)
   */
  private function setCurrentLanguage()
  {
    $lang = $this->session->getFromNamespace('controller', 'currentLanguage');
    if($this->request->getParameter($this->languageParameter) != '')
    {
      if(in_array($this->request->getParameter($this->languageParameter), array_keys($this->availableLanguages)))
        $lang = $this->request->getParameter($this->languageParameter);
    }
    if($lang == '')
    {
      $tok = strtok($_SERVER['HTTP_ACCEPT_LANGUAGE'], "[-,]");
      while ($lang == '')
      {
        list ($tok, $x) = explode(';', $tok);
        if(in_array($tok, array_keys($this->availableLanguages)))
        {
          $lang = $tok;
          if($lang != 'en' && strpos($lang, '_') === false)
            $lang .= '_' . strtoupper($lang);
          else
            $lang = 'en_US';
        }
        $tok = strtok("[-,]");
        if($tok === FALSE)
          $lang = $this->defaultLanguage;
      }
    }

    if(strpos($lang, '.UTF-8') === FALSE)
      $lang .= '.UTF-8';

    if($this->session->getFromNamespace('controller', 'currentLanguage') != $lang)
    {
      myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Using %s as application language", $lang), ALERT_DEBUG);
      $this->session->setInNamespace('controller', 'currentLanguage', $lang);
    }
    $x = setlocale(LC_ALL, $lang);
    bindtextdomain($this->getTextLocaleDomain, $this->getTextLocaleDir);
    textdomain($this->getTextLocaleDomain);
  }

  /**
   * Process the request.
   * Action or View will be evaluated.
   * 
   * @throws myMVC_MVCException             On problems
   */
  public function process()
  {
    $class = NULL;
    $type = NULL;

    if($this->availableLanguages !== NULL)
      $this->setCurrentLanguage();

    if($this->request->isEmptyRequest())
    {
      if($this->defaultAction !== NULL)
      {
        $class = $this->defaultAction;
        $type = 'ACTION';
      }
      else
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'No given action and no default action');
        throw new myMVC_MVCException($msg);
      }
    }
    else
    {
      if(($c = $this->request->getAction()) !== false)
      {
        $class = $c;
        $type = 'ACTION';
      }
      elseif(($c = $this->request->getView()) !== false)
      {
        $class = $c;
        $type = 'VIEW';
      }
      else
      {
        myMVC_Logger::doLog(__METHOD__, __LINE__, 'Unknown query. Trying default action', ALERT_WARNING);
        if($this->defaultAction !== NULL)
        {
          $class = $this->defaultAction;
          $type = 'ACTION';
        } else
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'No given action and no default action');
          throw new myMVC_MVCException($msg);
        }
      }
    }
    switch ($type)
    {
      case 'ACTION' :
        if($this->filters[self::$FILTER_PREACTION] !== NULL)
        {
          if(!in_array($class, $this->filters[self::$FILTER_PREACTION]['exceptions']))
          {
            $callClass = new $this->filters[self::$FILTER_PREACTION]['class']($this->request,
                                                                              $this->session,
                                                                              $this->actions[$class],
                                                                              self::$FILTER_PREACTION,
                                                                              $this->filters[self::$FILTER_PREACTION]['params']);
            $resp = $callClass->filter();
            if($resp !== NULL)
            {
              header("LOCATION: ".$resp->getNextURL());
              exit;
            }
          }
          else
            myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Action class %s in the exclude list of the filter. Skipping filtering", $class), ALERT_DEBUG);
        }

        $this->session->emptyGivenNamespace('FORMCHECK');
        $this->session->emptyGivenNamespace('MVCRESPONSE-'.$class);
        $this->session->deleteFromNamespace('MVCSAVEDFIELDS', 'fieldsValid');
        $this->session->deleteFromNamespace('MVCSAVEDFIELDS', 'fieldsInvalid');
        if(!isset($this->actions[$class]))
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Not existing action %s", $class));
          throw new myMVC_MVCException($msg);
        }
        $callClass = new $this->actions[$class]($this->request, $this->session);
        $userRights = $this->session->getFromNamespace('controller', 'userRights');
        if(!$userRights[get_class($callClass)])
        {
          if($this->noRightAction !== NULL)
          {
            header("LOCATION: ".myMVC_HttpResponse::createURL(myMVC_HttpRequest::$TYPE_ACTION, $this->noRightAction));
            exit;
          }
          else
          {
            $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'No right for action and no action to signal that');
            throw new myMVC_MVCException($msg);
          }
        }
        $this->session->setInNamespace('controller', 'previousAction', $this->session->getFromNamespace('controller', 'currentAction'));
        $this->session->setInNamespace('controller', 'currentAction', $class);
        $this->session->setInNamespace('controller', 'currentActionClass', $this->actions[$class]);
        if($callClass->validate())
          $resp = $callClass->perform();
        else
          $resp = $callClass->manageInvalidData();
        $this->session->setInNamespace('MVCSAVEDFIELDS', 'allFields', $this->request->getAllParameters(false));
        $this->session->setInNamespace('MVCSAVEDFIELDS', 'fieldsValid', $this->request->getAllParameters(true));
        $this->session->setInNamespace('MVCSAVEDFIELDS', 'fieldsInvalid', $this->request->getInvalidParameters());
        $this->session->setInNamespace('MVCRESPONSE-'.$class, 'menu', ($this->request->getQueryType() == 'menu'));
        header("LOCATION: ".$resp->getNextURL());
        exit;
        break;
      case 'VIEW' :
        if($this->filters[self::$FILTER_PREVIEW] !== NULL)
        {
          if(!in_array($class, $this->filters[self::$FILTER_PREVIEW]['exceptions']))
          {
            $callClass = new $this->filters[self::$FILTER_PREVIEW]['class']($this->request,
                                                                              $this->session,
                                                                              $this->views[$class],
                                                                              self::$FILTER_PREVIEW,
                                                                              $this->filters[self::$FILTER_PREVIEW]['params']);
            $resp = $callClass->filter();
            if($resp !== NULL)
            {
              header("LOCATION: ".$resp->getNextURL());
              exit;
            }
          }
          else
            myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("View class %s in the exclude list of the filter. Skipping filtering", $class), ALERT_DEBUG);
        }

        if(!isset($this->views[$class]))
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Not existing view %s", $class));
          throw new myMVC_MVCException($msg);
        }
        if($this->renderEngine === NULL)
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'No render engine specified');
          throw new myMVC_MVCException($msg);
        }
        $callClass = new $this->views[$class]($this->renderEngine, $this->session);
        $this->session->setInNamespace('controller', 'previousView', $this->session->getFromNamespace('controller', 'currentView'));
        $this->session->setInNamespace('controller', 'currentView', $class);
        $this->session->setInNamespace('controller', 'currentViewClass', $this->views[$class]);
        if($callClass->preRender())
          $callClass->render();
        else
        {
          $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("%s::preRender() returned false. Unable to display page", $class));
          throw new myMVC_MVCException($msg);
        }
        break;
    }
  }
}
?>
