<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Manager for the responses (next page)
 */
class myMVC_HttpResponse
{
  private $nextClass;
  private $nextIsDefault;
  private $type;
  private $arguments;

  /**
   * Constructor
   */
  public function __construct()
  {
    $this->nextClass = NULL;
    $this->nextIsDefault = false;
    $this->type = myMVC_HttpRequest::$TYPE_UNKNOWN;
    $this->arguments = array();
  }

  /**
   * Set the next Action as the given
   * default Action
   */
  public function setNextDefaultAction()
  {
    $session = myMVC_HttpSession::getInstance();
    $traces = debug_backtrace();
    $caller = $traces[1]['class'];
    $session->emptyGivenNamespace('MVCRESPONSE-'.$caller);
    $this->nextIsDefault = true;
  }

  /**
   * Set next Action or View
   * 
   * @param Class $class            The class of the next action or view
   * @throws myMVC_MVCException     On Problems
   */
  public function setNext($class)
  {
    $session = myMVC_HttpSession::getInstance();

    $traces = debug_backtrace();
    $caller = $traces[1]['class'];
    $session->emptyGivenNamespace('MVCRESPONSE-'.$caller);
    if(is_subclass_of($class, 'myMVC_BaseAction'))
    {
      if(!is_callable(array($class, 'getActionToken')))
        throw new myMVC_MVCException(sprintf("Action class %s has no method getActionToken", $class));
      $this->nextClass = call_user_func(array($class, 'getActionToken'));
      $this->type = myMVC_HttpRequest::$TYPE_ACTION;
    }
    else if(is_subclass_of($class, 'myMVC_BaseView'))
    {
      if(!is_callable(array($class, 'getViewToken')))
        throw new myMVC_MVCException(sprintf("View class %s has no method getViewToken", $class));
      $this->nextClass = call_user_func(array($class, 'getViewToken'));
      $this->type = myMVC_HttpRequest::$TYPE_VIEW;
    }
    else
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__,
              sprintf("Class %s is either an action nor a view", $class));
      throw new myMVC_MVCException($msg);
    }
  }

  /**
   * Create the URL for the next page, using the given class (Action or View)
   * and the given parameters
   * 
   * @return string                 The next URL
   */
  public function getNextURL()
  {
    $defaultAction = myMVC_Controller::getDefaultAction();
    if($this->nextIsDefault)
    {
      $class = $defaultAction;
      myMVC_Logger::doLog(__METHOD__, __LINE__, 'Redirecting to default action', ALERT_DEBUG);
    }
    else
      $class = $this->nextClass;
    
    return self::createURL($this->type, $class, $this->arguments);
  }

  /**
   * Create the URL for a given Action or View, using the given parameters
   * 
   * @param string $type            Type of the next page (Action or View)
   * @param string $token           Token of the next page (for the actions returned from getActionToken,
   *                                                        for the views, from getViewToken)
   * @param array $args             The optional arguments
   * @return string                 The next URL
   */
  public static function createURL($type, $token, $args = NULL)
  {
    global $config;

    $params = '';
    if(is_array($args))
      foreach(array_keys($args) as $key)
        $params .= sprintf("&%s=%s", $key, $args[$key]);
    $sessId = '';
    if($config['SESSION_USECOOKIES'] == 0)
    {
      $session = myMVC_HttpSession::getInstance();
      $sessId = '&'.$session->getSessionId();
    }
    return sprintf("?%s=%s%s%s", $type, $token, $params, $sessId);
  }

  /**
   * Save a variable to be passed as parameter for the next page (Action or View)
   * 
   * @param string $key             The name of the parameter
   * @param string $value           The value
   */
  public function setParameterForNextPage($key, $value)
  {
    $this->arguments[$key] = $value;
  }
}
?>
